--  This code is for demo of MATrix
--  1)  It has Multiplier in 89C51, the operands are given to 89C51
--      from S7-S4 & S3-S0, the result is displayed on 7 segment display
--  2)  It also takes relay control i/ps from S8 & S9 and triggers the relays
--      on relay card.
--  3)  This code also puts some character on dot matrix rolling display.
--  4)  Displays running LED on L15 - L0
--  Using FPGA of MATrix 

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity MATrix_Demo is
    Port ( 
            clk     : in std_logic;     -- 8 MHz clock i/p
            rst     : in std_logic;     -- system reset

            Rel1_in : in std_logic;     -- Relay 1 trigger I/P
            Rel2_in : in std_logic;     -- Relay 2 trigger I/P
            Rel1    : out std_logic;    -- Relay 1 O/P
            Rel2    : out std_logic;    -- Relay 2 o/P

            Stop    : in std_logic;     -- I/P to stop rolling of display
           	R 		: out std_logic_vector(6 downto 0);		-- rows or data
            C		: out std_logic_vector(4 downto 0);	     -- columns

            Led     : out std_logic_vector(15 downto 0);	-- LED o/ps

			a 		: in std_logic_vector(3 downto 0);		-- operand A for multiplier
           	b 		: in std_logic_vector(3 downto 0);		-- operand B for multiplier
          	p0_L	: out std_logic_vector(3 downto 0);		-- PORT 0 operand o/p  ( Lower)
           	p0_U	: out std_logic_vector(3 downto 0);		-- PORT 0 operand o/p  (Upper)
           	p2_seg	: in std_logic_vector(6 downto 0);		-- Result from port 2
			en_in	: in std_logic_vector(3 downto 0);      -- Display control
			en		: out std_logic_vector(3 downto 0);     
           	seg		: out std_logic_vector(6 downto 0));	-- result o/p 7 segments
end MATrix_Demo;

architecture Behavioral of MATrix_Demo is
-- Dot matrix rolling display component
component rolling_Disp
    Port ( 
			col_clk	: in std_logic;
   			row_clk : in std_logic;
            Stop    : in std_logic; 
			Rst		: in std_logic;
           	R 		: out std_logic_vector(6 downto 0);		-- row or data
            C		: out std_logic_vector(4 downto 0));	-- columns
end component;
	COMPONENT running_led
	PORT(
		LED_clk : IN std_logic;
		rst : IN std_logic;          
		LED : OUT std_logic_vector(15 downto 0)
		);
	END COMPONENT;
signal  div			: std_logic_vector(24 downto 0);
signal row_clk_s    : std_logic;
signal col_clk_s    : std_logic;
signal led_clk_s    : std_logic;
begin

-- Component  instantiation for Dot matrix rolling display
	u1: rolling_disp PORT MAP(
		col_clk =>col_clk_s ,
		row_clk =>row_clk_s ,
        Stop => stop,
		Rst =>rst ,
		R =>r ,
		C =>c 
	);

-- component for running LEDs
	u2: running_led PORT MAP(
		LED_clk => led_clk_s,
		rst => rst,
		LED => led
	);

-- Relay Trigger signals
Rel1    <= Rel1_in;
Rel2    <= Rel2_in;

-- Multipler in 89c51 I/Os
p0_L	<= a;
p0_U	<= b;
seg		<= p2_seg;
en		<= en_in;

-------------------------- clock divider  -----------------------
	process(clk,rst)
	begin
		if rst='1' then
			div	<= (others=>'0');
		elsif clk'event and clk='1' then
			div	<= div + 1;
		end if;
	end process;
------------------------------------------------------------------------
col_clk_s   <= div(12);
row_clk_s   <= div(20);
LED_clk_s   <= div(18);

end Behavioral;
