---- Program to test rolling display
--  MATrix code
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity rolling_Disp is
    Port ( 
			col_clk : in std_logic;
			row_clk : in std_logic;
			Rst		: in std_logic;
			Stop    : in std_logic;
           	R 		: out std_logic_vector(6 downto 0);		-- row or data
            C		: out std_logic_vector(4 downto 0));	-- columns
end rolling_Disp;

architecture Behavioral of rolling_Disp is
signal	row_s		: std_logic_vector(6 downto 0);		-- data generator
signal	col_s		: std_logic_vector(4 downto 0);		-- column selector
signal	c0,c1,c2	: std_logic_vector(6 downto 0);		-- data wrt columns
signal	c3,c4		: std_logic_vector(6 downto 0);		-- data wrt columns
signal	cntr		: std_logic_vector(4 downto 0);		-- offset
signal	add			: std_logic_vector(4 downto 0);		-- Address generator
signal	add_s		: std_logic_vector(4 downto 0);		-- subtractor

begin

------------------------------------------------------------------------
	process(col_clk,rst)
	begin
		if rst='1' then
			col_s	<= (others=>'0');
		elsif col_clk'event and col_clk='0' then
                if col_s = "10011" then
                    col_s	<= (others=>'0');
                else
        			col_s	<= col_s + 1;
    	        end if;
		end if;
	end process;

C	<= col_s;
------------------------------------------------------------------------

-- Address generator  ----
	process(row_clk,rst,stop)
	begin
		if rst='1' then
			cntr	<= (others=>'0');
		elsif row_clk'event and row_clk='1' then
            if stop='1' then
                if cntr="10101" then
        			cntr	<= (others=>'0');
                else
			     	cntr	<= cntr + 1;
        		end if;
            end if;
		end if;
	end process;
------------------------------------------------------------------------

r			<= row_s;

add_s		<=  "00010" when col_s = "00000" else    -- 0
				"00011" when col_s = "00001" else    -- 1       
				"00100" when col_s = "00010" else    -- 2
				"00101" when col_s = "00011" else    -- 3
				"00110" when col_s = "00100" else    -- 4
                "00111" when col_s = "00101" else    -- 5
                "01000" when col_s = "00110" else    -- 6
                "01001" when col_s = "00111" else    -- 7
                "01010" when col_s = "01000" else    -- 8
                "01011" when col_s = "01001" else    -- 9
                "01100" when col_s = "01010" else    -- 10
                "01101" when col_s = "01011" else    -- 11
                "01110" when col_s = "01100" else    -- 12
                "01111" when col_s = "01101" else    -- 13
                "10000" when col_s = "01110" else    -- 14
                "10001" when col_s = "01111" else    -- 15
                "10010" when col_s = "10000" else    -- 16
                "10011" when col_s = "10001" else    -- 17
                "10100" when col_s = "10010" else    -- 18
                "10101" when col_s = "10011" else    -- 19
                "00010";

--			C0 C1 C2 C3 C4 C5 C6 C7 C8 C9 C10 C11 C12 C13 C14 C15 C16 C17 C18 C19
--		R0
--		R1
--		R2
--		R3
--		R4
--		R5
--		R6

add	<= add_s - cntr;
--
--for char "bvp"
--D6 - d0
--R7 - R1 

-- with add select
-- row_s<=  "0000000" when "00000",
-- 		 "0000000" when "00001",
-- 		 "0011111" when "00010",    --B
-- 		 "0010101" when "00011",    --B
-- 		 "0010101" when "00100",    --B
-- 		 "0010101" when "00101",    --B
-- 		 "0001010" when "00110",    --B
-- 		 "0000111" when "00111",    --V
--        "0001000" when "01000",    --V  
--          "0010000" when "01001",    --V
--          "0001000" when "01010",    --V
--          "0000111" when "01011",    --V  
--          "0011111" when "01100",    --P
--          "0000101" when "01101",    --P
--          "0000101" when "01110",    --P
--          "0000101" when "01111",    --P
--          "0000010" when "10000",    --P
--          "0000000" when others; 

 
 with add select
 row_s	<= 	"0000000" when "00000",
 		   	"0000000" when "00001",
 		   	"1111100" when "00010",    --n
 		   	"0000100" when "00011",    --n
 		   	"0000100" when "00100",    --n
 		   	"0000100" when "00101",    --n
 		   	"1111000" when "00110",    --n
 		   	"0000000" when "00111",    --i
         	"0000000" when "01000",    --i  
            "1111010" when "01001",    --i
            "0000000" when "01010",    --i
            "0000000" when "01011",    --i  
            "1000100" when "01100",    --2
            "1100010" when "01101",    --2
            "1010010" when "01110",    --2
            "1001100" when "01111",    --2
            "1000000" when "10000",    --2
            "0000000" when others; 

end Behavioral;  
