-------------------------------------------------------------------------------
-- Title				:	ADC DAC controller
-- Project			:	Sample code
-- File				:	adc_dac_matrix.vhd
-------------------------------------------------------------------------------
-- Organization	:	ni logic PVt. Ltd., Pune, India
-------------------------------------------------------------------------------
-- Description		:	controls the ADC in continuous mode and gives
--				data to DAC with dac_sel switch selection.
--				Also generates RAMP o/p and can be assigned to
--				DAC i/p with dac_sel switch
-------------------------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
-------------------------------------------------------------------------------
entity adc_dac_matrix is
    Port ( clk,rst      : in std_logic;
           dac_op       : out std_logic_vector(7 downto 0);
           ADC_in       : in std_logic_vector(7 downto 0);
           add          : out std_logic_vector(2 downto 0);
           eoc          : in std_logic;
			  dac_sel      : in std_logic;
           soc          : out std_logic;
           Ale          : out std_logic;
           Adc_Clk      : out std_logic;   -- samplinc clock
           ADC_OE       : out std_logic;   -- adc o/p enable
           adc_out      : out std_logic_vector(7 downto 0));
end adc_dac_matrix;

architecture Behavioral of adc_dac_matrix is
signal dac_clk	: std_logic;
signal led_clk	: std_logic;
signal eoc_s	: std_logic;
signal latch    : std_logic_vector(7 downto 0);
signal dac_s    : std_logic_vector(7 downto 0);
signal div 		: std_logic_vector(22 downto 0);
begin

-- dummy logic to bypass the BUFGP of FPGA
eoc_s	<= eoc and (not rst);

-- for continuous operation
soc         <= eoc;
adc_oe      <= '1';
ale         <= '1';
add         <= "000";
adc_clk     <= div(4);
dac_clk     <= div(6);

dac_op      <= latch when dac_sel = '0' else
					dac_s;
------------------------------------------------------------------------
    process(clk,rst)
    begin
    	if rst='1' then 
    		div	<= (others=>'0');
    	 elsif clk'event and clk='1' then
        	div <= div +1;
        end if ;
    end process;
------------------------------------------------------------------------- 
-- latching data coming from ADC 
    process(eoc_s,rst)
    begin
        if rst='1' then
            latch   <= (others=>'0');
        elsif eoc_s'event and eoc_s='1' then
            latch   <= adc_in;
        end if;
    end process;
-------------------------------------------------------------------------  
adc_out     <= latch;       --ADC o/ps
-------------------------------------------------------------------------  
-- ramp wave generation, simple 8-bit counter
    process(dac_clk,rst)
    begin
        if rst='1' then
            dac_s   <= (others=>'0');
        elsif dac_clk'event and dac_clk='1' then
            dac_s   <= dac_s + 1;
        end if;
    end process;




-------------------------------------------------------------------------  

end Behavioral;
