-------------------------------------------------------------------------------
-- Title				:	Rolling Display
-- Project			:	Sample code
-- File				:	rolling_new.vhd
-------------------------------------------------------------------------------
-- Organization	:	ni logic PVt. Ltd., Pune, India
-------------------------------------------------------------------------------
-- Description		:	DOT MATRIX ROLLLING DISPLAY SYSTEM WITH LEFT DIRECTION,AND
--							RIGHT DIRECTION ,STANDALONE AND BLINKING OPTIONS.
-------------------------------------------------------------------------------  
Library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity rolling_temp is
port(
	clk,rst: in std_logic;
	col_sel: out std_logic_vector( 4 downto 0);
	row_sel: out std_logic_vector( 6 downto 0));
end rolling_temp;

architecture behave of rolling_temp is
	signal clkdiv: std_logic_vector( 26 downto 0);-------------clock divider
	signal row_clk,col_clk,st_clk,bl_clk: std_logic;-----------rowclk,colmn clk,state transition clk and blinking speed clock
	signal state: std_logic_vector(1 downto 0); ---------------state 0 to 3
	signal decoder,decoder_s: std_logic_vector( 4 downto 0);---column selection decoder
	signal 	row_sel_s : std_logic_vector( 6 downto 0);--------7bit row data
	signal j,p: integer range 0 to 32;-------------------------data  rate counter 2 sent to row lines
	signal i: integer range 0 to  19; -------------------------column selection rate counter
	
	type shiftreg is array ( 0 to 13) of std_logic_vector(6 downto 0); --data  storage buffer	of 7 bit width
	signal shifter:  shiftreg :=(	
											"0111110",-----	|\	   |
 											"0010000",-----	| \	|	 
 											"0001000",-----	|  \	|
  											"0000100",-----	|   \	|
  											"0111110",-----	|    \|
											"0000000",-----      |			
											"0000000",-----	   |
											"0111110",-----		|
	 										"0000000",-----		|
											"0000000",-----		|
														 ---  	______	 
 											"0110010",-----   		|
											"1000110",-----	______|
											"1001010",-----  |
											"0110010" -----  |______
											 );
											
begin
  
col_clk<=clkdiv(9);
row_clk<=clkdiv(18);
bl_clk<=clkdiv(20);
st_clk<=clkdiv(24);

col_sel<= decoder_s;
row_sel<=shifter(p) when p<=13 else ( others=>'0');  

	-----CLOCK DIVIDER PROCESS     
	process(clk,rst)
	begin
		if rst='1' then
			clkdiv<=(others=>'0');
		elsif clk'event and clk='1' then
			clkdiv<=clkdiv+1;
		end if;
	end process;

----STATE INCREMENT PROCESS
				 
	process(st_clk,rst)
	begin
		if rst='1' then
			state<="00";
		elsif st_clk'event and st_clk='1' then
			state<=state+1;
		end if;
	end process;

-------	COLOMN DECODING PROCESS
	process(rst,col_clk)
	begin
		if rst='1' then
			decoder<=(others=>'0');
		elsif col_clk'event and col_clk='1' then
				if ( decoder="10011")  then
						decoder<=(others=>'0');
				else
						decoder<=decoder+1;
				end if;
		end if;
	end process;

------ ROW INCREMENTING PROCESS			
	process(rst,row_clk)
	begin
		if rst='1' then
			j<=0;
		elsif row_clk'event and row_clk='1' then
			j<= j+1;
  		end if;
	end process;


i	<= conv_integer(decoder);----VECTOR 2 INTEGER 



-----BLINKING STATE EXECUTION BY DISABLING DECODERLINES
	process(state,decoder)
	begin
		if not (state="11") then
				decoder_s<= decoder;
	 	elsif bl_clk='1' then
	 			decoder_s<=(others=>'1');
		 	else
	 			decoder_s<=decoder;
	 	end if;
	 end process;


-----ROW DATA ASSIGNING W.R.T STATE 
	with state select
		 p	<=	(i-j) 	when 	"01" , ----TOWARDS RIGHT
	   	 	(i+j) 	when 	"10" , ---TOWARDS LEFT
		 		i      	when others;	---STANDALONE  AND BLINK


 end architecture ;                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     