--------------------------------------------------------------------------------
-- Title				:	Sign counter
-- Project			:	Sample code
-- File				:	sign_cntr.vhd
-------------------------------------------------------------------------------
-- Organization	:	ni logic PVt. Ltd., Pune, India
-------------------------------------------------------------------------------
-- Description		:	4-bit sign counter to count in +/- direction with
--							overflow flags.
-------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity sign_cntr is
    Port ( clk,rst : in std_logic;
           dir : in std_logic;								--	Direction I/P
           pov : out std_logic;								-- Positive overflow
           nov : out std_logic;								-- Negative overflow
           sign : out std_logic;								-- Sign indication o/p
           mag : out std_logic_vector(3 downto 0));	-- Magnitude of counter o/p
end sign_cntr;

architecture Behavioral of sign_cntr is
signal mag_s	: std_logic_vector(3 downto 0);--------magnitude counter
signal sign_s	: std_logic;---------------------------sign bit display signal
signal pov_s	: std_logic;  -------------------------positve overflow display signal
signal nov_s	: std_logic;---------------------------negative overflow display signal
begin


---- Main updown counter
process(clk,rst)
begin
	if rst='1' then
	mag_s	<= (others=>'0');
	sign_s<='0';
	pov_s<='0';
	nov_s<='0';
	elsif clk'event and clk='1' then
		if dir='1' then        -- upcounter
			if mag_s ="1111" then	-- if it counting up from negative values ,,sign bit have2 become '0'
 		   		sign_s<='0' ;
			end if;	
				mag_s	<= mag_s + 1; -- countup by 1
			if sign_s='0' then				  
				if mag_s="1111" then
					pov_s<= '1';			---positive overflow occurs when count becomes "0000" from "1111", with sign='0'; 
				 else
					pov_s<='0';
			    end if;
			else
			   pov_s<='0';
			end if;
				
			else
	   		if mag_s ="0000" then 	----if it counting down from positive values ,,sign bit have2 become '1'
 		     		sign_s<='1' ;
				end if;

			mag_s	<= mag_s - 1;	---countdown by 1
				
			if sign_s='1'then
				if mag_s= "0001" then
			   	nov_s<= '1'; 		 	---negative overflow occurs when count becomes "0001(2'scomplement of "1111") from "0000", with sign='1'***; 
				else
					nov_s<='0';
				end if;
			else
			   nov_s<='0';
			end if;
	    	end if;
end if;
end process;


-- O/P assignments
mag <= mag_s(3 downto 0) when sign_s='0' else
          (not mag_s(3 downto 0)) + 1 ;---magnitude in 2'scomplement form if sign='1'

	pov	<=pov_s;
	nov	<=nov_s;
	sign	<=sign_s;

end Behavioral;



