--------------------------------------------------------------------------------
-- Title				:	Sign counter
-- Project			:	Sample code
-- File				:	sign_cntr_7seg.vhd
-------------------------------------------------------------------------------
-- Organization	:	ni logic PVt. Ltd., Pune, India
-------------------------------------------------------------------------------
-- Description		:	4-bit sign counter to count in +/- direction with
--							overflow flags.
--							Displays the counter o/p on the 7-segent displays.
-------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity sign_cntr_7seg is
    Port ( 	clk,rst : in std_logic;
           	dir : in std_logic;
           	pov : out std_logic;
           	nov : out std_logic;
			  	en : out std_logic_vector(2 downto 0);
          	seg : out std_logic_vector(6 downto 0));
end sign_cntr_7seg;

architecture Behavioral of sign_cntr_7seg is
signal clkdiv: std_logic_vector(21 downto 0); --------22 bit clock divide counter
signal clk_in,en_clk : std_logic;  -------------------main counter clk and 7segment enable counter clk 
signal mag_s	: std_logic_vector(3 downto 0);--------magnetude counter
signal mag     :  std_logic_vector(3 downto 0);-------magnetude display vector
signal sign_s	: std_logic;---------------------------sign bit display signal
signal pov_s	: std_logic;  -------------------------positve overflow display signal
signal nov_s	: std_logic;---------------------------negative overflow display signal
signal en_s    :  std_logic_vector(2 downto 0);	 -----7segment enable vector
signal seg_lsb :  std_logic_vector(6 downto 0); ------lower digit display segment
signal seg_msb :  std_logic_vector(6 downto 0); ------upper digit display segment
signal seg_sign :  std_logic_vector(6 downto 0); -----sign bit display segment

begin

-----------clock divider-----------
process( clk,rst)
begin
	if rst='1' then
		clkdiv<= (others=>'0');
	elsif clk'event and clk='1' then
		clkdiv<=clkdiv+1;
	end if;
end process;


-------clock assingment to acutal counter and 7-seg enable counter

clk_in<=clkdiv(21); ----counter clock
en_clk<=clkdiv(10); --seg enable clock
en<=en_s;




---- Main updown counter
process(clk_in,rst)
begin
	if rst='1' then
	mag_s	<= (others=>'0');
	sign_s<='0';
	pov_s<='0';
	nov_s<='0';
	elsif clk_in'event and clk_in='1' then
			if dir='1' then        ---upcounter
				if mag_s ="1111" then	----if it counting up from negative values ,,sign bit have2 become '0'
 		   		sign_s<='0' ;
				end if;	
			  	
			   mag_s	<= mag_s + 1; ---countup by 1
				
				if sign_s='0' then				  
					if mag_s="1111" then
						pov_s<= '1';			---positive overflow occurs when count becomes "0000" from "1111", with sign='0'; 
					 else
						pov_s<='0';
				    end if;
				else
				   pov_s<='0';
				
				end if;
				
			 else
	   		 if mag_s ="0000" then 	----if it counting down from positive values ,,sign bit have2 become '1'
 		     		sign_s<='1' ;
				 end if;
				 					
				mag_s	<= mag_s - 1;	---countdown by 1
				
				if sign_s='1'then
					if mag_s= "0001" then
				   	nov_s<= '1'; 		 	---negative overflow occurs when count becomes "0001(2'scomplement of "1111") from "0000", with sign='1'***; 
					
					else
						nov_s<='0';
					end if;
				else
				   nov_s<='0';
				end if;
	    	end if;
 end if;
end process;

	mag <= mag_s(3 downto 0) when sign_s='0' else
          (not mag_s(3 downto 0)) + 1 ;---magnetude in 2'scomplement form if sign='1'

	pov<=pov_s;
	nov<=nov_s;
	--sign<=sign_s;


 ----///////////////////////////7-SEGMENT DISPLAY SECTION///////////////----------
 
--------seg enable counter...
process(en_clk,rst)
begin
	if rst='1' then
	   en_s<="001";
		elsif en_clk'event and en_clk='1' then
		en_s<= (en_s(0) & en_s(2 downto 1));
	end if;
	end process;


 --------------seg selection through enable
 with en_s select
  seg<= seg_lsb when "100", ----en2
       seg_msb when "010",	 ----en1
		 seg_sign when "001",-----en0
		 "0000000" when others;
 
 ---- display of  lower digit  with 7-segment coding.
with  mag  select
--          abcdefg						--display number
  seg_lsb <="1111110" when "0000",   -- 0
            "0110000" when "0001",   -- 1    
            "1101101" when "0010",   -- 2
            "1111001" when "0011",   -- 3
            "0110011" when "0100",   -- 4
            "1011011" when "0101",   -- 5
            "1011111" when "0110",   -- 6
            "1110000" when "0111",   -- 7
            "1111111" when "1000",   -- 8
            "1111011" when "1001",   -- 9
            "1111110" when "1010",   -- 0
            "0110000" when "1011",   -- 1    
            "1101101" when "1100",   -- 2
            "1111001" when "1101",   -- 3
            "0110011" when "1110",   -- 4
            "1011011" when "1111",   -- 5
            "0000001" when others;


---display of MSD with 7-seg coding
with  mag  select
--          abcdefg						 -- display no:
  seg_msb <="1111110" when "0000",   -- 0
            "1111110" when "0001",   -- 0    
            "1111110" when "0010",   -- 0
            "1111110" when "0011",   -- 0
            "1111110" when "0100",   -- 0
            "1111110" when "0101",   -- 0
            "1111110" when "0110",   -- 0
            "1111110" when "0111",   -- 0
            "1111110" when "1000",   -- 0
            "1111110" when "1001",   -- 0
            "0110000" when "1010",   -- 1
            "0110000" when "1011",   -- 1    
            "0110000" when "1100",   -- 1
            "0110000" when "1101",   -- 1
            "0110000" when "1110",   -- 1
            "0110000" when "1111",   -- 1
            "0110000" when others;


---sign bit display with 7seg-coding
	with  sign_s  select
										  --sign
	seg_sign<="0000000" when '0',	---0
	          "0000001" when '1',	---1
				 "0000000" when others;

  -------------------------------------------------

end Behavioral;



